#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
# A script to run MR and refinement on a cluster node. Acts as a stadn-alone executable.
# Ronan Keegan - 31/1/07
#

import os, sys, string
import shutil

import molrepEXE
import phaserEXE
import refmacEXE
import pdbtools

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 not found'
if not os.environ.has_key('CBIN'):
    raise RuntimeError, 'CBIN not found'


class ClusterJob:
   """ A class to handle the running of MR and refinement on a cluster. """

   def __init__(self):
     
      self.mr_program=""
      self.working_DIR=""

      self.hklin_SG=""
      self.hklin_SG_Code=""
      self.enant_SG_Code=""

      self.hklin1=""
      self.hklin2=""
      self.refine_hklin=""
      self.phaser_hklout=""

      self.mr_log=""

      self.pdbin_list=[]
      self.pdbout=""
      self.pdbout1=""
      self.pdbout2=""

      self.refine_hklout=""
      self.refine_pdbout=""
      self.refine_log=""

      self.ENANT=False

      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

      self.local_debug = False
      

   def parse_input(self, input_file):
      """ A function to parse the input file for the parameters and filenames for MR and refinement. """

      # Check that input file exists
      if os.path.isfile(input_file) == False:
         sys.stdout.write("Cluster Log: Input parameter file not found\n")
         sys.stdout.write("\n")
      # Read the input parameters according to their four letter keys
      else:
         infile=open(input_file, "r")
         line=string.split(infile.readline())

         while line:
            if "DIRE" in line[0].upper():
               self.working_DIR=line[1]
            if "SGIN" in line[0].upper():
               self.hklin_SG=line[1]
            if "HKL1" in line[0].upper():
               self.hklin1=line[1]
            if "HKL2" in line[0].upper():
               self.hklin2=line[1]
            if "HKLR" in line[0].upper():
               self.refine_hklin=line[1]
            if "HKLO" in line[0].upper():
               self.phaser_hklout=line[1]
            if "PDBO" in line[0].upper():
               self.pdbout=line[1]
            if "MRLO" in line[0].upper():
               self.mr_log=line[1]
            if "REFH" in line[0].upper():
               self.refine_hklout=line[1]
            if "REFP" in line[0].upper():
               self.refine_pdbout=line[1]
            if "REFL" in line[0].upper():
               self.refine_log=line[1]
            if "ENAN" in line[0].upper():
               if line[1].upper() == "TRUE":
                  self.ENANT=True
               else:
                  self.ENANT=False

            # An arbitrary number of PDB model files may be passed in so we need to reconstruct
            # this list of files. For Molrep 1st one in list is model
            if "PDBI" in line[0].upper():
               for filename in line[1:]: 
                  self.pdbin_list.append(filename)
        
            line=string.split(infile.readline())

         infile.close()

   def run(self, mr_program, refine_program, mr_keyfile, refine_keyfile):
      """ A function to run MR and refinement on a cluster node. """

      # Change directory to the mr directory for this model
      os.chdir(os.path.join(self.working_DIR, "mr"))

      if mr_program.upper() == "MOLREP":   
         self.mr_program = "MOLREP"

         # Instantiate the class
         molrep_job1=molrepEXE.MolrepEXE()

         molrep_job1.local_debug=True

         # Determine the spacegroup code for the enantiomorph
         if self.ENANT:
            self.hklin_SG_Code=molrep_job1.SG_Codes[self.hklin_SG]   
            self.pdbout1 = os.path.join(self.working_DIR, "mr", "molrep_pdbout_" + self.hklin_SG_Code + ".pdb")
            # Set the log file for this run of molrep
            molrep1_log=os.path.join(self.working_DIR, "mr", "molrep1_" + self.hklin_SG_Code + ".log")
         else:
            self.pdbout1 = os.path.join(self.working_DIR, "mr", "molrep_pdbout.pdb")
            # Set the log file for this run of molrep
            molrep1_log=os.path.join(self.working_DIR, "mr", "molrep1.log")

         # Run Molrep
         molrep_job1.run(os.path.join(self.working_DIR, "mr"), self.hklin1, self.pdbin_list[0], self.pdbout1, molrep1_log, keyfile=mr_keyfile)

         # Extract contrast value for this job
         molrep_job1.check_contrast()

         # Search in the enantiomorphic spacegroup
         if self.ENANT:
            self.enant_SG_Code=molrep_job1.ENANT_SG[self.hklin_SG_Code]
            self.pdbout2 = os.path.join(self.working_DIR, "mr", "molrep_pdbout_" + self.enant_SG_Code + ".pdb")
      
            # Instantiate the class
            molrep_job2=molrepEXE.MolrepEXE()
      
            molrep_job2.local_debug=True
      
            # Check that NOSG line is not already in the mr keyword file
            file=open(mr_keyfile, "r")
            line=file.readline()
            found=False
            while line:
               if "NOSG" in string.split(line)[0].upper(): 
                  found=True
               line=file.readline()
            file.close()

            # Update the keyword input file to search in enantiomorphic SG
            if found==False:
               file=open(mr_keyfile, "a")
               file.write("NOSG " + self.enant_SG_Code)
               file.close()
       
            # Set the log file for this run of molrep
            molrep2_log=os.path.join(self.working_DIR, "mr", "molrep2_" + self.enant_SG_Code + ".log")

            # Run Molrep in enantiomorphic SG, translation function only 
            molrep_job2.run(os.path.join(self.working_DIR, "mr"), self.hklin1, self.pdbin_list[0], \
                                         self.pdbout2, molrep2_log, mode="T", keyfile=mr_keyfile)
      
            # Extract contrast value for this job
            molrep_job2.check_contrast()

            print "Final contrast for HKLIN : %.3f" % molrep_job1.contrast
            print "Final contrast for Enant : %.3f" % molrep_job2.contrast
       
            if molrep_job1.contrast >= molrep_job2.contrast:
               shutil.copy(self.pdbout1, self.pdbout)   
               shutil.copy(self.hklin1, self.refine_hklin)   
               shutil.copy(molrep1_log, self.mr_log)   
            else:
               shutil.copy(self.pdbout2, self.pdbout)   
               shutil.copy(self.hklin2, self.refine_hklin)   
               shutil.copy(molrep2_log, self.mr_log)   
         else:
            print "Final contrast for HKLIN : %.3f" % molrep_job1.contrast
            shutil.copy(self.pdbout1, self.pdbout)   
            shutil.copy(self.hklin1, self.refine_hklin)   
            shutil.copy(molrep1_log, self.mr_log)   

      elif mr_program.upper() == "PHASER":   
         self.mr_program = "PHASER"

         # Instantiate the class
         phaser_job=phaserEXE.PhaserEXE()

         phaser_job.local_debug=True

         # Run Phaser
         phaser_job.run(self.phaser_hklout, self.pdbout, self.mr_log, mr_keyfile)

         # Check the spacegroup of the Phaser solution
         phaser_job.check_SG(self.phaser_hklout)

         # Report the spacegroup of the solution and setup the input MTZ file for refinement
         sys.stdout.write("MR log: Spacegroup of solution from Phaser is: %s\n" % phaser_job.soln_spacegroup)
         if string.strip(phaser_job.soln_spacegroup).replace(" ","").lower() \
               == string.strip(self.hklin_SG).replace(" ","").lower():
            sys.stdout.write("        - this is the spacegroup of the input target MTZ file\n")

            # Copy the input MTZ file to the MTZ file to be processed in refinement
            shutil.copyfile(self.hklin1, self.refine_hklin)

         else:
            sys.stdout.write("        - this is the enantiomorphic spacegroup of the target data\n")

            # Copy the reindexed input MTZ file to the MTZ file to be processed in refinement
            shutil.copyfile(self.hklin2, self.refine_hklin)

         sys.stdout.write("\n")

      # Get the number of molecules found
      details=pdbtools.Get_PDB_Details()
      details.get_details(self.pdbout)
     
      # Create summary of the PDB file using pdbcur
      #details.pdbcur(pdbcur_logfile, pdbout)
      
      sys.stdout.write("Number of molecules found in %s : %d\n" % (self.pdbout, details.mol_count))
      sys.stdout.write("\n")
      
      # If using Phaser set up a Phaser job
      # Setup a refmac job
      refmac_job=refmacEXE.RefmacEXE()

      refmac_job.local_debug = True
 
      # Run refmac
      refmac_job.run(self.refine_hklin, self.refine_hklout, self.pdbout, self.refine_pdbout, self.refine_log, refine_keyfile)

if __name__ == "__main__":
   """ An example run. Also allows this function to be called as an executable on clusters."""

   # Check and parse command line arguments
   if len(sys.argv) == 6:
      input_file     = sys.argv[1]
      mr_keyfile     = sys.argv[2]
      refine_keyfile = sys.argv[3]
      mr_program     = sys.argv[4]
      refine_program = sys.argv[5]
   else:
      sys.stdout.write("Usage: python cluster_run.py <input_file> <mr_keyfile> <refine_keyfile> <mr_program> <refine_program>\n")
      sys.stdout.write("\n")
      sys.exit()

   # Setup a cluster job
   cljob=ClusterJob()

   # Read the input parameters for MR and refinement
   cljob.parse_input(input_file)

   # Launch the clusterised job
   cljob.run(mr_program, refine_program, mr_keyfile, refine_keyfile)
   
