#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
# A MrBUMP wrapper for Phaser 
# Ronan Keegan - 29/1/07
#

import os, sys
import string
import time

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 not found'
if not os.environ.has_key('CBIN'):
    raise RuntimeError, 'CBIN not found'

class PhaserEXE:
   """ A class to run a Phaser job. """

   def __init__(self):
      if os.name=="nt":
         self.phaserEXE  = os.path.join(os.environ["CBIN"], "phaser.exe")  
         self.mtzdumpEXE = os.path.join(os.environ["CBIN"], "mtzdump.exe")  
      else:
         self.phaserEXE  = os.path.join(os.environ["CBIN"], "phaser")  
         self.mtzdumpEXE = os.path.join(os.environ["CBIN"], "mtzdump")  

      self.logfile = ""
      self.hklout = ""
      self.pdbout = ""
      self.key = ""
      self.keywords = []
      self.termination=False
      self.soln_spacegroup=""
      self.soln_found=False

      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

      self.local_debug = False

   def set_logfile(self, filename):
      self.logfile=filename

   def set_hklout(self, filename):
      self.hklout=filename

   def set_pdbout(self, filename):
      self.pdbout=filename

   def add_keyword(self, keyword):
      self.keywords.append(keyword)
      

   def check_SG(self, mtzfile):
      """ Check the spacegroup of the Phaser solution from the output MTZ file. 
          hklout   - output mtz file from Phaser,
          hklinSG - the spacegroup of the input MTZ file"""

      # Run mtzdump to dump the file details

      # Check the file exists
      if os.path.isfile(mtzfile):

         # Set the command line
         command_line=self.mtzdumpEXE + " HKLIN " + mtzfile
   
         # Open a pipe to the job
         i,oe=os.popen4(command_line)
   
         # Write the keywords to the job
         i.write("GO")
         i.close()
   
         # Watch the output for successful termination
         out=oe.readline()
   
         while out:
            if self.local_debug:
               sys.stdout.write(out)
   
            if 'Space group' in out:
               self.soln_spacegroup=string.split(out,"'")[1] 
   
            out=oe.readline()
   
         oe.close()

      else:
         # Report an error
         sys.stdout.write("Phaser output MTZ error: file not found\n")
         sys.stdout.write("\n")

   def run(self, hklout, pdbout, logfile, keyfile=""):
      """ Function to launch a phaser job. """

      # Give a header output
      sys.stdout.write("#######################################\n")
      sys.stdout.write("Running Phaser...... \n")
      sys.stdout.write("#######################################\n")
      sys.stdout.write("\n")

      # Check to see if the logfile has been specified, otherwise give it a default name
      if self.logfile=="":
         self.logfile=logfile 

      # Set HKLOUT if it has not been set already
      if self.hklout=="":
         self.hklout=hklout

      # Set PDBOUT 
      if self.pdbout=="":
         self.pdbout=pdbout

      # Set the keywords for the job 
      if keyfile == "":
      # If no keyword file is given read keywords from the input keywords
         for keyword in self.keywords:
            self.key=self.key + keyword + "\n"

      else:
      # Read keywords from the keyword file 
         kfile=open(keyfile, "r")
         line=kfile.readline()
         while line:
            self.key = self.key + line
            line=kfile.readline()
         kfile.close()
        
      # Output the keywords if in debug mode
      if self.local_debug:
         sys.stdout.write("#########################\n")
         sys.stdout.write("Phaser keyword input:\n")
         sys.stdout.write("#########################\n")
         sys.stdout.write("\n")
         sys.stdout.write(self.key)
         sys.stdout.write("\n")
         time.sleep(2)
         
      # Set the command line
      command_line=self.phaserEXE

      # Open a pipe to the job
      i,oe=os.popen4(command_line)

      # Write the keywords to the job
      i.write(self.key)
      i.close()

      # Open the log file for writing
      log=open(self.logfile, "w")

      # Watch the output for successful termination
      out=oe.readline()
      log.write(out)

      if self.debug:
         sys.stdout.write(out)

      while out:
         if self.local_debug:
            sys.stdout.write(out)

         if 'EXIT STATUS: FAILURE' in out:
            self.termination=False

         out=oe.readline()
         log.write(out)

      oe.close()
      log.close()

      # Check the output

      # Check for the output files:
      if os.path.exists(self.pdbout):
         self.termination = True
      else:
         self.termination = False

      # If this is an anisotropic correction run of phaser only an MTZ is produced.
      if "MODE MR_ANO" in self.key:
         if os.path.exists(self.hklout):
            self.termination = True
         else:
            self.termination = False

      # Report the result
      if self.termination == False: 
         self.soln_found=False
         sys.stdout.write("Phaser Error: phaser failed to complete successfully\n")
         sys.stdout.write("Log file can be found here:\n")
         sys.stdout.write("   " + self.logfile + "\n")
         sys.stdout.write("\n")
      else:
         self.soln_found=True
         sys.stdout.write("Phaser completed successfully\n")
         sys.stdout.write("Output Log file written to:\n   " + self.logfile + "\n")
         sys.stdout.write("Output MTZ file written to:\n   " + self.hklout + "\n")
         if "MODE MR_ANO" not in self.key:
            sys.stdout.write("Output PDB file written to:\n   " + self.pdbout + "\n")
         sys.stdout.write("\n")

